From ce92ba767a9557daf7f18be94882dd7e6f4591fb Mon Sep 17 00:00:00 2001
From: alecpl <alec@alec.pl>
Date: Thu, 30 Sep 2010 09:24:33 -0400
Subject: [PATCH] - Plugin API: improved 'abort' flag handling, added 'result' item in some hooks: group_*, contact_*, identity_* (#1486914)
---
program/include/rcube_message.php | 143 ++++++++++++++++++++++++++++++++++++-----------
1 files changed, 110 insertions(+), 33 deletions(-)
diff --git a/program/include/rcube_message.php b/program/include/rcube_message.php
index 124120b..4351f17 100644
--- a/program/include/rcube_message.php
+++ b/program/include/rcube_message.php
@@ -4,8 +4,8 @@
+-----------------------------------------------------------------------+
| program/include/rcube_message.php |
| |
- | This file is part of the RoundCube Webmail client |
- | Copyright (C) 2008-2010, RoundCube Dev. - Switzerland |
+ | This file is part of the Roundcube Webmail client |
+ | Copyright (C) 2008-2010, Roundcube Dev. - Switzerland |
| Licensed under the GNU GPL |
| |
| PURPOSE: |
@@ -29,7 +29,18 @@
*/
class rcube_message
{
+ /**
+ * Instace of rcmail.
+ *
+ * @var rcmail
+ */
private $app;
+
+ /**
+ * Instance of imap class
+ *
+ * @var rcube_imap
+ */
private $imap;
private $opt = array();
private $inline_parts = array();
@@ -98,8 +109,8 @@
/**
* Return a (decoded) message header
*
- * @param string Header name
- * @param bool Don't mime-decode the value
+ * @param string $name Header name
+ * @param bool $row Don't mime-decode the value
* @return string Header value
*/
public function get_header($name, $raw = false)
@@ -112,7 +123,7 @@
/**
* Set is_safe var and session data
*
- * @param bool enable/disable
+ * @param bool $safe enable/disable
*/
public function set_safe($safe = true)
{
@@ -124,7 +135,7 @@
/**
* Compose a valid URL for getting a message part
*
- * @param string Part MIME-ID
+ * @param string $mime_id Part MIME-ID
* @return string URL or false if part does not exist
*/
public function get_part_url($mime_id)
@@ -139,8 +150,8 @@
/**
* Get content of a specific part of this message
*
- * @param string Part MIME-ID
- * @param resource File pointer to save the message part
+ * @param string $mime_id Part MIME-ID
+ * @param resource $fp File pointer to save the message part
* @return string Part content
*/
public function get_part_content($mime_id, $fp=NULL)
@@ -214,23 +225,22 @@
if ($mimetype == 'text/plain') {
$out = $this->imap->get_message_part($this->uid, $mime_id, $part);
-
+
// re-format format=flowed content
if ($part->ctype_secondary == 'plain' && $part->ctype_parameters['format'] == 'flowed')
$out = self::unfold_flowed($out);
break;
}
else if ($mimetype == 'text/html') {
- $html_part = $this->imap->get_message_part($this->uid, $mime_id, $part);
+ $out = $this->imap->get_message_part($this->uid, $mime_id, $part);
// remove special chars encoding
$trans = array_flip(get_html_translation_table(HTML_ENTITIES));
- $html_part = strtr($html_part, $trans);
+ $out = strtr($out, $trans);
// create instance of html2text class
- $txt = new html2text($html_part);
+ $txt = new html2text($out);
$out = $txt->get_text();
- break;
}
}
@@ -242,13 +252,13 @@
* Raad the message structure returend by the IMAP server
* and build flat lists of content parts and attachments
*
* @param object rcube_message_part Message structure node
* @param bool True when called recursively
*/
private function parse_structure($structure, $recursive = false)
{
// real content-type of message/rfc822 part
- if ($mimetype == 'message/rfc822' && $structure->real_mimetype)
+ if ($structure->mimetype == 'message/rfc822' && $structure->real_mimetype)
$mimetype = $structure->real_mimetype;
else
$mimetype = $structure->mimetype;
@@ -393,6 +403,17 @@
else if ((($part_mimetype == 'text/plain' || $part_mimetype == 'text/html') && $mail_part->disposition != 'attachment') ||
$part_mimetype == 'message/delivery-status' || $part_mimetype == 'message/disposition-notification'
) {
+ // Allow plugins to handle also this part
+ $plugin = $this->app->plugins->exec_hook('message_part_structure',
+ array('object' => $this, 'structure' => $mail_part,
+ 'mimetype' => $part_mimetype, 'recursive' => true));
+
+ if ($plugin['abort'])
+ continue;
+
+ $mail_part = $plugin['structure'];
+ list($primary_type, $secondary_type) = explode('/', $plugin['mimetype']);
+
// add text part if it matches the prefs
if (!$this->parse_alternative ||
($secondary_type == 'html' && $this->opt['prefer_html']) ||
@@ -491,7 +512,7 @@
/**
* Fill aflat array with references to all parts, indexed by part numbers
*
- * @param object rcube_message_part Message body structure
+ * @param rcube_message_part $part Message body structure
*/
private function get_mime_numbers(&$part)
{
@@ -507,7 +528,8 @@
/**
* Decode a Microsoft Outlook TNEF part (winmail.dat)
*
- * @param object rcube_message_part Message part to decode
+ * @param rcube_message_part $part Message part to decode
+ * @return array
*/
function tnef_decode(&$part)
{
@@ -542,7 +564,8 @@
/**
* Parse message body for UUencoded attachments bodies
*
- * @param object rcube_message_part Message part to decode
+ * @param rcube_message_part $part Message part to decode
+ * @return array
*/
function uu_decode(&$part)
{
@@ -588,34 +611,88 @@
/**
* Interpret a format=flowed message body according to RFC 2646
*
- * @param string Raw body formatted as flowed text
+ * @param string $text Raw body formatted as flowed text
* @return string Interpreted text with unwrapped lines and stuffed space removed
*/
public static function unfold_flowed($text)
{
- return preg_replace(
- array('/-- (\r?\n)/', '/^ /m', '/(.) \r?\n/', '/--%SIGEND%(\r?\n)/'),
- array('--%SIGEND%\\1', '', '\\1 ', '-- \\1'),
- $text);
+ $text = preg_split('/\r?\n/', $text);
+ $last = -1;
+ $q_level = 0;
+
+ foreach ($text as $idx => $line) {
+ if ($line[0] == '>' && preg_match('/^(>+\s*)/', $line, $regs)) {
+ $q = strlen(str_replace(' ', '', $regs[0]));
+ $line = substr($line, strlen($regs[0]));
+
+ if ($q == $q_level && $line
+ && isset($text[$last])
+ && $text[$last][strlen($text[$last])-1] == ' '
+ ) {
+ $text[$last] .= $line;
+ unset($text[$idx]);
+ }
+ else {
+ $last = $idx;
+ }
+ }
+ else {
+ $q = 0;
+ if ($line == '-- ') {
+ $last = $idx;
+ }
+ else {
+ // remove space-stuffing
+ $line = preg_replace('/^\s/', '', $line);
+
+ if (isset($text[$last]) && $line
+ && $text[$last] != '-- '
+ && $text[$last][strlen($text[$last])-1] == ' '
+ ) {
+ $text[$last] .= $line;
+ unset($text[$idx]);
+ }
+ else {
+ $text[$idx] = $line;
+ $last = $idx;
+ }
+ }
+ }
+ $q_level = $q;
+ }
+
+ return implode("\r\n", $text);
}
/**
* Wrap the given text to comply with RFC 2646
+ *
+ * @param string $text Text to wrap
+ * @param int $length Length
+ * @return string Wrapped text
*/
public static function format_flowed($text, $length = 72)
{
- $out = '';
-
- foreach (preg_split('/\r?\n/', trim($text)) as $line) {
- // don't wrap quoted lines (to avoid wrapping problems)
- if ($line[0] != '>')
- $line = rc_wordwrap(rtrim($line), $length - 1, " \r\n");
+ $text = preg_split('/\r?\n/', $text);
- $out .= $line . "\r\n";
+ foreach ($text as $idx => $line) {
+ if ($line != '-- ') {
+ if ($line[0] == '>' && preg_match('/^(>+)/', $line, $regs)) {
+ $prefix = $regs[0];
+ $level = strlen($prefix);
+ $line = rtrim(substr($line, $level));
+ $line = $prefix . rc_wordwrap($line, $length - $level - 2, " \r\n$prefix ");
+ }
+ else if ($line) {
+ $line = ' ' . rc_wordwrap(rtrim($line), $length - 2, " \r\n ");
+ }
+
+ $text[$idx] = $line;
+ }
}
-
- return $out;
+
+ return implode("\r\n", $text);
}
}
@@ -242,13 +252,13 @@
* and build flat lists of content parts and attachments
*
- * @param object rcube_message_part Message structure node
- * @param bool True when called recursively
+ * @param rcube_message_part $structure Message structure node
+ * @param bool $recursive True when called recursively
*/
private function parse_structure($structure, $recursive = false)
--
Gitblit v1.9.1